package com.sanjeev.deals.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Table;

@Entity
@Table(name = "ITEMS")

public class Item implements Serializable {
	
	@Id @GeneratedValue(strategy=GenerationType.IDENTITY)
    private Long id;
	
	@Column(name = "ITEM_NAME")
	private String name;
	
	@Column(name = "MODEL_NUMBER")
	private String modelNumber;
	
	@Column(name = "MANUFACTURER")
	private String manufacturer;
	
	@Column(name = "ITEM_CATEGORY")
	private String category;
	
	@org.hibernate.annotations.CollectionOfElements (fetch = FetchType.EAGER)
	@JoinTable(
			name = "ITEM_RETAILER_INFO",
			joinColumns = @JoinColumn(name = "id")
			)
	private Collection<RetailerItemInfo> retailers = new ArrayList<RetailerItemInfo>();
	/**
	 * @return the id
	 */
	public Long getId() {
		return id;
	}
	/**
	 * @param id the id to set
	 */
	public void setId(Long id) {
		this.id = id;
	}
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return the modelNumber
	 */
	public String getModelNumber() {
		return modelNumber;
	}
	/**
	 * @param modelNumber the modelNumber to set
	 */
	public void setModelNumber(String modelNumber) {
		this.modelNumber = modelNumber;
	}
	/**
	 * @return the manufacturer
	 */
	public String getManufacturer() {
		return manufacturer;
	}
	/**
	 * @param manufacturer the manufacturer to set
	 */
	public void setManufacturer(String manufacturer) {
		this.manufacturer = manufacturer;
	}
	/**
	 * @return the category
	 */
	public String getCategory() {
		return category;
	}
	/**
	 * @param category the category to set
	 */
	public void setCategory(String category) {
		this.category = category;
	}
	/**
	 * @return the retailers
	 */
	public Collection<RetailerItemInfo> getRetailers() {
		return retailers;
	}
	/**
	 * @param retailers the retailers to set
	 */
	public void setRetailers(List<RetailerItemInfo> retailers) {
		this.retailers = retailers;
	}
	
	public void addRetailer(RetailerItemInfo retailer) {
		boolean addRetailer = true;
		//Check if this retailer-item combo already exists, and is a live deal.
		if(retailers.size() > 0){
			for(RetailerItemInfo _retailer : retailers) {
				if(_retailer.getRetailerName().equals(retailer.getRetailerName())) {
					//we found a retailer-item combo from the same retailer.
					//find if this item is a expired deal
					if(_retailer.getDealExpirationDate().after(new Date())) {
						//deal has not expired, please do not add
						addRetailer = false;
						//since we have already decided not to add this item, no need to iterate any further
						break;
					}
				}
			}
		}

		if(addRetailer) {
			//set the item of this retailer to item
			retailer.setItem(this);
			retailers.add(retailer);
		}
	}
	
	/**
	 * This is a helper method, to be used when parsing and persisting items.
	 * @return
	 */
	public RetailerItemInfo getSingleRetailer() {
		RetailerItemInfo retailerItemInfo = null;
		for (RetailerItemInfo _retailerItemInfo : retailers) {
			retailerItemInfo = _retailerItemInfo;
			break;
		}
		return retailerItemInfo;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Item [");
		if (category != null) {
			builder.append("category=");
			builder.append(category);
			builder.append(", ");
		}
		if (id != null) {
			builder.append("id=");
			builder.append(id);
			builder.append(", ");
		}
		if (manufacturer != null) {
			builder.append("manufacturer=");
			builder.append(manufacturer);
			builder.append(", ");
		}
		if (modelNumber != null) {
			builder.append("modelNumber=");
			builder.append(modelNumber);
			builder.append(", ");
		}
		if (name != null) {
			builder.append("name=");
			builder.append(name);
			builder.append(", ");
		}
		if (retailers != null) {
			builder.append("retailers=");
			builder.append(retailers);
		}
		builder.append("]");
		return builder.toString();
	}

}
